#version 450

#extension GL_ARB_separate_shader_objects : enable
#extension GL_ARB_shading_language_420pack : enable

layout (location = 0) in vec4 inPos;
layout (location = 1) in vec3 inNormal;
layout (location = 2) in vec2 inUV;
layout (location = 3) in vec3 inColor;

layout (binding = 0) uniform UBO 
{
	mat4 projection;
	mat4 modelView;
	mat4 invModelView;
	vec4 cameraPosition;
	vec2 framebuffer_size;
	int envprobe;
} ubo;

struct Material {
	vec4 albedo;
	vec4 A;
	vec4 mu_o;
	vec4 mu_e;
	float eta_o;
	float eta_e;
	float _unused0, _unused1;
};
layout (binding = 2) uniform Params {
	Material material;
	float exposure;
	float polarization;
	float d;
} params;

layout (location = 0) out vec3 outColor;
layout (location = 1) out vec3 outPos;
layout (location = 2) out vec2 outUV;
layout (location = 3) out vec3 outNormal;
layout (location = 4) out vec3 outPosW;

void main() 
{
	const vec3 P = vec3(inPos.xy/vec2(1.1,1.5), inPos.z*params.d*1e-3f);
	const vec4 transformed = ubo.modelView * vec4(P, 1);
	const vec3 screenSpaceNormal = mat3(ubo.invModelView) * inNormal;

	outColor = inColor;
	outPos = transformed.xyz;
	outUV = inUV;
	outNormal = screenSpaceNormal;
	outPosW = P;
	
	gl_Position = ubo.projection * transformed;	
}
